/*
 * TaskDelegateMixin.hh
 *
 * Copyright (c) 2003, 2004 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/**
 * @file TaskDelegateMixin.hh
 *
 * Default implementations for the Broker::Task methods.  Kind of a hack
 * really, but it works.
 *
 * @sa RealTimeTaskDelegateImpl.hh
 */

/** @copydoc Broker::Task::Name */
virtual char *Name(void)
    throw (CORBA::SystemException)
{
    if( CORBA::is_nil(this->dm_RemoteObject.in()) )
    {
	throw CORBA::BAD_INV_ORDER();
    }
    return this->dm_RemoteObject->Name();
};

/** @copydoc Broker::Task::SetManager */
virtual void SetManager(Broker::Manager_ptr man)
    throw (CORBA::SystemException)
{
    if( CORBA::is_nil(this->dm_RemoteObject.in()) )
    {
	throw CORBA::BAD_INV_ORDER();
    }
    this->dm_RemoteObject->SetManager(man);
};

/** @copydoc Broker::Task::BeginCPUScheduling */
virtual void BeginCPUScheduling(const Broker::ScheduleParameters &sp)
    throw (CORBA::SystemException,
	   Broker::DuplicateScheduleParameter,
	   Broker::InvalidScheduleParameter,
	   Broker::MissingScheduleParameter)
{
    if( CORBA::is_nil(this->dm_RemoteObject.in()) )
    {
	throw CORBA::BAD_INV_ORDER();
    }
    this->dm_RemoteObject->BeginCPUScheduling(sp);
};

/** @copydoc Broker::Task::EndCPUScheduling */
virtual void EndCPUScheduling(void)
    throw (CORBA::SystemException)
{
    if( CORBA::is_nil(this->dm_RemoteObject.in()) )
    {
	throw CORBA::BAD_INV_ORDER();
    }
    this->dm_RemoteObject->EndCPUScheduling();
};
