/*
 * GlacialTaskAdvocate.hh
 *
 * Copyright (c) 2003, 2004 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/**
 * @file GlacialTaskAdvocate.hh
 *
 * Header file for the GlacialTaskAdvocate class.
 */

#ifndef _glacial_task_advocate_hh
#define _glacial_task_advocate_hh

#include <RealTimeTaskDelegateImpl.hh>

/**
 * An advocate that is slow to react to changes in the CPU usage.  This
 * advocate is mostly useful for applications that have a relatively flat usage
 * pattern with an infrequent spike in usage.
 *
 * The current implementation increases or decreases its advice by a quarter of
 * the difference between the last received advice and the current.
 */
class GlacialTaskAdvocate : public virtual RealTimeTaskDelegateImpl
{

public:

    /** Default constructor. */
    GlacialTaskAdvocate(void);

    /** Destructor. */
    virtual ~GlacialTaskAdvocate(void);

    /** @copydoc Broker::RealTimeTask::PassCPU */
    virtual Broker::CPUReserve PassCPU(Broker::RealTimeTask_ptr rtt,
				       const Broker::CPUReserve &status,
				       const Broker::CPUReserve &advice,
				       const Broker::KeyedReportParameters &krp)
	throw (CORBA::SystemException);

private:

    /**
     * The last advice reported by this advocate.
     */
    CORBA::ULong gta_LastAdvice;

};

#endif
